program tiled_csv;
{
  This code is dedicated to the public domain, and comes with no warranty of any
  kind. This code has every possibility to be absolutely horrible, and I believe
  that a year from now I will think of it in this way. As such, please take most
  of my coding practices and skill level into consideration, and as this is
  public domain you are free to improve and release this into the public under
  any license of your choosing.
   - Matt Shaffer (http://matt-shaffer.com) aka dazappa

  P.S. make sure you go into the project options to change the path to ZGL
  P.P.S. I am using dynamic linking because static linking is kind of broken
  under Lazarus .9.30, and I would rather not explain a few dozen steps to get
  it to compile statically here (which involves changing files).

  * If you are converting this to compile under FPC only, you shouldn't have
  much trouble. Rename this file to a .pas, and compile with
  -Fu(YOURZGLDIR)/headers/

  *** The font files are NOT (afaik) licensed in the public domain. They are a
  part of the default ZenGL demos. *** (All other images were created by me,
  and therefore I release them into the public domain as well)

  This code only tested on: Lazarus .9.30, FPC 2.4.2, ZenGL .2.1, Tiled QT .6
  ----------------------------
  Program tiled_csv:
  This program provides a quick example for skipping through the XML and directly
  loading the CSV layer data (and rendering it) only.
}
{$IFDEF WINDOWS}
  {$R *.res}
{$ENDIF}

uses
  zglHeader, SysUtils {for some string handling}, Classes {for TStringList}
  ;

type
  TTile = class
    public
      index: integer;
  end;

  TTileMap = class
    public
      tile: array of TTile;
  end;

var
  f: zglPFont;
  tex: zglPTexture;
  mapW, mapH: integer;
  layer: array of TTileMap;

procedure debug(output: string);
begin
  WriteLn(output);
end;

procedure loadMap(fname: string);
var
  mapfile: TextFile;
  s, s2, beginning: string;
  llen, i: integer;
  csv: TStringList;
begin
  AssignFile(mapfile, fname);
  Reset(mapfile);

  while(not(EOF(mapfile))) do
  begin
    ReadLn(mapfile, s);
    s := LowerCase(Trim(s));

    beginning := LeftStr(s, 5);

    if(beginning = '<map ') then
    begin
      // Do some basic XML parsing / hacking to get dimensions of map

      // Sets s2 = the string starting from the width
      s2 := Copy(s, Pos('width', s)+7, 10000);
      // Final parsing, copying to the double quote, we now have a number
      mapW := StrToInt(Copy(s2, 1, Pos('"', s2)-1));

      // Sets s2 = the string starting from the width
      s2 := Copy(s, Pos('height', s)+8, 10000);
      // Final parsing, copying to the double quote, we now have a number
      mapH := StrToInt(Copy(s2, 1, Pos('"', s2)-1));

      debug('Map dimensions: '+IntToStr(mapW)+'x'+IntToStr(mapH));
    end else
    if(beginning = '<laye') then
    begin
      // Wee, we have a new tile layer full of delicious CSV tile data

      // Initialize objects and arrays to the map dimensions
      llen := Length(layer); // Going to be using this a lot, so make a var
      SetLength(layer, llen+1);
      layer[llen] := TTileMap.Create;
      SetLength(layer[llen].tile, mapW * mapH);
      for i := 0 to mapW * mapH do
      begin
        layer[llen].tile[i] := TTile.Create;
      end;
      debug('layer '+IntToStr(llen)+' objects initialized');

      // Read until we hit the CSV data
      while(not(s = '<data encoding="csv">')) do // This is the last line before
      begin
        ReadLn(mapfile, s);
        s := LowerCase(Trim(s));
      end;

      csv := TStringList.Create;
      s2 := '';
      // Read CSV data until no more
      while(not(s = '</data>')) do
      begin
        ReadLn(mapfile, s);
        s := LowerCase(Trim(s));
        s2 := Concat(s2, s);
      end;
      s2 := Copy(s2, 1, Length(s2)-7); // </data> would otherwise be appended
      debug(s2);

      // CSV split into a TStringList
      csv.StrictDelimiter := true;
      csv.Delimiter := ',';
      csv.DelimitedText := s2;
      debug('-----');

      // Tile data populated
      for i := 0 to csv.Count-1 do
      begin
        layer[llen].tile[i].index := StrToInt(csv[i]);
      end;

      debug('');
    end;
  end;

  CloseFile(mapfile);
end;

procedure Init;
begin
  f := font_LoadFromFile('font.zfi');
  tex := tex_LoadFromFile('tiles.png', $FF000000, TEX_DEFAULT_2D);
  tex_SetFrameSize(tex, 32, 32);
  loadMap('map_csv.tmx');
end;

procedure Draw;
var
  i, i2, i3, c, c2: integer;
begin
  batch2d_Begin();
  // Disabling the color buffer will improve performance, but as this is a demo
  // project we shouldn't be worried too much. Supplying this demo project an
  // invalid file, for instance, might also result in no images being drawn,
  // which would mean the font for the FPS would be unreadable.
  //zgl_Disable(COLOR_BUFFER_CLEAR);

  // This starts with index 1 for the frame, Tiled starts at 1 as well, with 0
  // being no tile
  // So anyway, yeah tile drawing starts here
  {
    Tiled is pretty nice. See, your lowest level layers are put first in the file,
    (ex: ground is the lowest layer and first in the file), so it is very darn
    convenient for drawing, as for layering you have to draw the lowest layer
    first.
  }
  c := 0;
  c2 := 0;
  for i := 0 to mapH-1 do
  begin
    for i2 := 0 to mapW-1 do
    begin
      for i3 := 0 to Length(layer)-1 do
      begin
        if(not(layer[i3].tile[c].index = 0)) then
        begin
          // tex, x, y, w, h, angle, index
          asprite2d_Draw(tex, i2*32, i*32, 32, 32, 0, layer[i3].tile[c].index);
          Inc(c2);
        end;
      end;
      Inc(c);
    end;
  end;

  text_Draw(f, 0, 0, 'fps: ' + u_IntToStr(zgl_Get(RENDER_FPS)));
  text_draw(f, 0, 26, 'tiles: '+ u_IntToStr(c2));
  batch2d_End();
end;

begin
  zglLoad(libZenGL);

  randomize();

  zgl_Reg(SYS_LOAD, @Init);
  zgl_Reg(SYS_DRAW, @Draw);

  zgl_Enable(APP_USE_UTF8);

  wnd_SetCaption('Tiled CSV Example');

  wnd_ShowCursor(TRUE);

  scr_SetOptions(640, 480, REFRESH_MAXIMUM, FALSE, FALSE);

  zgl_Init();
end.

